/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: DrlPackageAdder.java 4855 2009-08-10 12:59:59Z loris $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.rules.osgi.impl;

import java.io.Reader;
import java.io.StringReader;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.drools.KnowledgeBase;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.osgi.framework.Bundle;
import org.ow2.jasmine.rules.osgi.api.IPackageAdder;

/**
 *
 *
 */
public class DrlPackageAdder extends AbstractPackageAdder
                             implements IPackageAdder {

    // -------------------------------------------------------------------------
    // Variables
    // -------------------------------------------------------------------------

    /**
     * Logger.
     */
    private static Logger logger =
        Logger.getLogger(DrlPackageAdder.class.getName());

    /**
     * The Drools package name
     */
    private final String packageName;

    /**
     * The Drl file contents
     */
    private final String drlFile;


    // -------------------------------------------------------------------------
    // Constructors
    // -------------------------------------------------------------------------

    /**
     * Create a new package adder to add rules belonging to a given bundle
     * into a working memory.
     *
     * @param bundle the bundle containing rules
     * @param globals the globals
     * @param the targeted working memory name
     */
    public DrlPackageAdder(final Bundle bundle,
                           Map<String, Object> globals,
                           final String packageName,
                           final String drlFile) {

        super(bundle, globals);
        this.packageName = packageName;
        this.drlFile = drlFile;
    }

    /**
     * Create a new package adder to add rules belonging to a given bundle
     * into the default working memory.
     *
     * @param bundle the bundle containing rules
     * @param globals the globals
     */
    public DrlPackageAdder(final Bundle bundle,
                           Map<String, Object> globals,
                           final String workingMemoryName,
                           final String packageName,
                           final String drlFile) {

        super(bundle, globals, workingMemoryName);
        this.packageName = packageName;
        this.drlFile = drlFile;
    }


    // -------------------------------------------------------------------------
    // Getters & Setters
    // -------------------------------------------------------------------------

    /**
     * Get the Drools package name
     */
    public String getPackageName() {
        return this.packageName;
    }


    // -------------------------------------------------------------------------
    // Implementation of IPackageAdder
    // -------------------------------------------------------------------------

    /**
     * Add some knowledge packages into the provided {@link KnowledgeBase}, by using the
     * given {@link KnowledgeBuilder}.
     *
     * @param builder a knowledge builder
     * @param knowledgeBase a knowledge base
     */
    public void addPackage(final KnowledgeBuilder builder, final KnowledgeBase knowledgeBase) {

        Reader drlFileReader = new StringReader(drlFile);
        builder.add(ResourceFactory.newReaderResource(drlFileReader), ResourceType.DRL);
        if (builder.hasErrors()) {
              logger.log(Level.WARNING, builder.getErrors().toString());
        }
        logger.log(Level.INFO, "Drl file loaded");

        knowledgeBase.addKnowledgePackages(builder.getKnowledgePackages());
    }

}
